﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Collections;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  [Item("DataTableHelper", "Helper class for creating datatables.")]
  [StorableClass]
  public class DataTableHelper : Item {

    [Storable]
    private ResultCollection resultsCol;
    [Storable]
    private string chartName;
    [Storable]
    private string[] dataRowNames;
    [Storable]
    DataTable dt;

    [StorableConstructor]
    private DataTableHelper(bool deserializing) : base(deserializing) { }
    private DataTableHelper(DataTableHelper original, Cloner cloner)
      : base(original, cloner) { }
    public DataTableHelper() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new DataTableHelper(this, cloner);
    }

    public void InitializeChart(ResultCollection results, string chartName, string[] dataRowNames) {
      if (!results.ContainsKey(chartName)) {
        this.resultsCol = results;
        this.chartName = chartName;
        this.dataRowNames = dataRowNames;

        dt = new DataTable(chartName);

        foreach (string dataRowName in dataRowNames) {
          DataRow dtRow = new DataRow(dataRowName);
          dt.Rows.Add(dtRow);
        }

        results.Add(new Result(chartName, dt));
      }
    }

    public void AddPoint(string dataRow, double point) {
      dt.Rows[dataRow].Values.Add(point);
    }

    public void AddPoint(double point) {
      // assume that there is only 1 row and therefore the name doesn't have to be provided
      dt.Rows[dataRowNames[0]].Values.Add(point);
    }

    public ObservableList<double> GetFirstDataRow() {
      return dt.Rows[dataRowNames[0]].Values;
    }

    public void CleanUp() {
      //remove chart
      resultsCol[chartName].Value = null;
      resultsCol.Remove(chartName);
      dt = null;
    }

    public void CleanUpAndCompressData() {
      string[] columnNames = new string[] { "Count", "Minimum", "Maximum", "Average", "Median", "Standard Deviation", "Variance", "25th Percentile", "75th Percentile", "Gradient", "Relative Error", "Avg. of Upper 25 %", " Avg. of Lower 25 %", "Avg. of First 25 %", "Avg. of Last 25 %" };

      foreach (string rowName in dataRowNames) {
        DataRow curDataRow = dt.Rows[rowName];
        var values = curDataRow.Values.AsEnumerable();

        double cnt = values.Count();
        double min = values.Min();
        double max = values.Max();
        double avg = values.Average();
        double median = values.Median();
        double stdDev = values.StandardDeviation();
        double variance = values.Variance();
        double percentile25 = values.Percentile(0.25);
        double percentile75 = values.Percentile(0.75);
        double k, d, r;
        LinearLeastSquaresFitting.Calculate(values.ToArray(), out k, out d);
        r = LinearLeastSquaresFitting.CalculateError(values.ToArray(), k, d);
        double lowerAvg = values.OrderBy(x => x).Take((int)(values.Count() * 0.25)).Average();
        double upperAvg = values.OrderByDescending(x => x).Take((int)(values.Count() * 0.25)).Average();
        double firstAvg = values.Take((int)(values.Count() * 0.25)).Average();
        double lastAvg = values.Skip((int)(values.Count() * 0.75)).Average();

        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[0], new DoubleValue(cnt)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[1], new DoubleValue(min)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[2], new DoubleValue(max)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[3], new DoubleValue(avg)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[4], new DoubleValue(median)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[5], new DoubleValue(stdDev)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[6], new DoubleValue(variance)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[7], new DoubleValue(percentile25)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[8], new DoubleValue(percentile75)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[9], new DoubleValue(k)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[10], new DoubleValue(r)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[11], new DoubleValue(upperAvg)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[12], new DoubleValue(lowerAvg)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[13], new DoubleValue(firstAvg)));
        resultsCol.Add(new Result(chartName + " " + rowName + " " + columnNames[14], new DoubleValue(lastAvg)));
      }

      CleanUp();
    }
  }
}
